__author__ = 'DarkWeb'

# Here, we are importing the auxiliary functions to clean or convert data
from MarketPlaces.Utilities.utilities import *

# Here, we are importing BeautifulSoup to search through the HTML tree
from bs4 import BeautifulSoup


# This is the method to parse the Description Pages (one page to each Product in the Listing Pages)
def ares_description_parser(soup):

    # Fields to be parsed

    vendor = "-1"                   # 0 *Vendor_Name
    success = "-1"                  # 1 Vendor_Successful_Transactions
    rating_vendor = "-1"            # 2 Vendor_Rating
    name = "-1"                     # 3 *Product_Name
    describe = "-1"                 # 4 Product_Description
    CVE = "-1"                      # 5 Product_CVE_Classification (Common Vulnerabilities and Exposures)
    MS = "-1"                       # 6 Product_MS_Classification (Microsoft Security)
    category = "-1"                 # 7 Product_Category
    views = "-1"                    # 8 Product_Number_Of_Views
    reviews = "-1"                  # 9 Product_Number_Of_Reviews
    rating_item = "-1"              # 10 Product_Rating
    addDate = "-1"                  # 11 Product_AddedDate
    BTC = "-1"                      # 12 Product_BTC_SellingPrice
    USD = "-1"                      # 13 Product_USD_SellingPrice
    EURO = "-1"                     # 14 Product_EURO_SellingPrice
    sold = "-1"                     # 15 Product_QuantitySold
    left = "-1"                     # 16 Product_QuantityLeft
    shipFrom = "-1"                 # 17 Product_ShippedFrom
    shipTo = "-1"                   # 18 Product_ShippedTo

    # Finding Product Name
    name = soup.find('div', {'class': "col-md-12 my-2"}).text
    name = name.replace('\n', ' ')
    name = name.replace(",", "")
    name = name.strip()

    bae = soup.find('div', {'class': "col-md-7"}).find('span').find_all('span')

    # Finding Vendor
    vendor = bae[0].text
    vendor = vendor.replace(",", "")
    vendor = vendor.replace("...", "")
    vendor = vendor.strip()

    # Finding Vendor Rating
    full_stars = bae[2].find_all('i', {'class': "fas fa-star"})
    half_star = bae[2].find('i', {'class': "fas fa-star-half-alt"})
    rating_vendor = len(full_stars) + (0.5 if half_star is not None else 0)

    # Finding Successful Transactions
    success = bae[4].text
    success = success.replace("Sales ", "")
    success = success.strip()

    bae = soup.find('span', {'class': "text-left"}).find_all('span')

    # Finding Prices
    USD = bae[0].text
    USD = USD.replace("\n$", "")
    USD = USD.strip()

    shipping_info = bae[4].text
    if "Digital" not in shipping_info:
        shipping_info = shipping_info.split("  ")

        # Finding Shipment Information (Origin)
        shipFrom = shipping_info[0].strip()

        # Finding Shipment Information (Destination)
        shipTo = shipping_info[1].strip()

    bae = soup.find_all('textarea')

    # Finding the Product description
    describe = bae[0].text
    describe = describe.replace("\n", " ")
    describe = describe.replace("\r", " ")
    describe = describe.strip()

    # Finding the Terms and Conditions
    terms = bae[1].text
    terms = terms.replace("\n", " ")
    terms = terms.strip()

    '''
    # Finding the Number of Product Reviews
    tag = soup.findAll(text=re.compile('Reviews'))
    for index in tag:
        reviews = index
        par = reviews.find('(')
        if par >=0:
            reviews = reviews.replace("Reviews (","")
            reviews = reviews.replace(")","")
            reviews = reviews.split(",")
            review = str(abs(int(reviews[0])) + abs(int(reviews[1])))
        else  :
            review = "-1"
    '''

    # Searching for CVE and MS categories
    cve = soup.findAll(text=re.compile('CVE-\d{4}-\d{4}'))
    if cve:
        CVE = " "
        for idx in cve:
            CVE += (idx)
            CVE += "  "
            CVE = CVE.replace(',', ' ')
            CVE = CVE.replace('\n', '')
    ms = soup.findAll(text=re.compile('MS\d{2}-\d{3}'))
    if ms:
        MS = " "
        for im in ms:
            MS += (im)
            MS += " "
            MS = MS.replace(',', ' ')
            MS = MS.replace('\n', '')

    # Populating the final variable (this should be a list with all fields scraped)
    row = (vendor, rating_vendor, success, name, describe, CVE, MS, category, views, reviews, rating_item, addDate,
           BTC, USD, EURO, sold, left, shipFrom, shipTo)

    # Sending the results
    return row


# This is the method to parse the Listing Pages
def ares_listing_parser(soup):

    # Fields to be parsed
    nm = 0                                  # *Total_Products (Should be Integer)
    mktName = "Ares"                        # 0 *Marketplace_Name
    vendor = []                             # 1 *Vendor
    rating_vendor = []                      # 2 Vendor_Rating
    success = []                            # 3 Vendor_Successful_Transactions
    name = []                               # 4 *Product_Name
    CVE = []                                # 5 Product_CVE_Classification (Common Vulnerabilities and Exposures)
    MS = []                                 # 6 Product_MS_Classification (Microsoft Security)
    category = []                           # 7 Product_Category
    describe = []                           # 8 Product_Description
    views = []                              # 9 Product_Number_Of_Views
    reviews = []                            # 10 Product_Number_Of_Reviews
    rating_item = []                        # 11 Product_Rating
    addDate = []                            # 12 Product_AddDate
    BTC = []                                # 13 Product_BTC_SellingPrice
    USD = []                                # 14 Product_USD_SellingPrice
    EURO = []                               # 15 Product_EURO_SellingPrice
    sold = []                               # 16 Product_QuantitySold
    qLeft = []                              # 17 Product_QuantityLeft
    shipFrom = []                           # 18 Product_ShippedFrom
    shipTo = []                             # 19 Product_ShippedTo
    href = []                               # 20 Product_Links

    listing = soup.findAll('div', {"class": "col-md-4 my-md-0 my-2 col-12"})

    # Populating the Number of Products
    nm = len(listing)

    for a in listing:
        bae = a.findAll('a', href=True)

        # Adding the url to the list of urls
        link = bae[0].get('href')
        link = cleanLink(link)
        href.append(link)

        # Finding the Vendor
        vendor_name = bae[1].text
        vendor_name = vendor_name.replace(",", "")
        vendor_name = vendor_name.strip()
        vendor.append(vendor_name)

        # Finding the Product
        product = bae[2].find('img').get('alt')
        product = product.replace('\n', ' ')
        product = product.replace(",", "")
        product = product.strip()
        name.append(product)

        # Searching for CVE and MS categories
        cve = a.findAll(text=re.compile('CVE-\d{4}-\d{4}'))
        if not cve:
            cveValue="-1"
        else:
            cee = " "
            for idx in cve:
                cee += (idx)
                cee += "  "
                cee = cee.replace(',', ' ')
                cee = cee.replace('\n', '')
            cveValue=cee
        CVE.append(cveValue)
        
        ms = a.findAll(text=re.compile('MS\d{2}-\d{3}'))
        if not ms:
            MSValue="-1"
        else:
            me = " "
            for im in ms:
                me += (im)
                me += " "
                me = me.replace(',', ' ')
                me = me.replace('\n', '')
            MSValue=me
        MS.append(MSValue)

    # Populate the final variable (this should be a list with all fields scraped)
    return organizeProducts(mktName, nm, vendor, rating_vendor, success, name, CVE, MS, category, describe, views,
                            reviews, rating_item, addDate, BTC, USD, EURO, sold, qLeft, shipFrom, shipTo, href)


def ares_links_parser(soup):

    # Returning all links that should be visited by the Crawler
    href = []

    listing = soup.findAll('a', {"class": "btn btn-success w-100 my-1"})

    for a in listing:

        link = a['href']
        href.append(link)

    return href